import logging
import os


class CustomFormatter(logging.Formatter):
    grey = "\\x1b[38;21m"
    yellow = "\\x1b[33;21m"
    red = "\\x1b[31;21m"
    bold_red = "\\x1b[31;1m"
    reset = "\\x1b[0m"
    format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"

    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: grey + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


class ServerConfig:
    def __init__(self):
        self.server = None

    def init_app(self, app):
        self.server = app

    def config_value(self, key):
        return self.server.config[key]


# WithPano Server Config Initialized
wpconfig = ServerConfig()

project_data_base_path = os.path.normpath(os.path.join(os.path.dirname(__file__), f"../../../project_data"))
